﻿using System;
using System.IO;
using System.Windows;
using System.Windows.Input;
using System.ComponentModel;
using Microsoft.Win32;
using Vlc.DotNet.Core;
using Vlc.DotNet.Core.Medias;
using Vlc.DotNet.Wpf;
using System.Runtime.InteropServices;
using System.Threading.Tasks;//2017.10.01 ADD
using System.Reflection;

namespace CuteMP4
{
    public partial class MainWindow : Window
    {
        //Version
        private readonly System.IO.FileInfo myInfo = new System.IO.FileInfo
            (
                Path.GetFileName(Assembly.GetExecutingAssembly().Location)
            );

        private readonly System.Diagnostics.FileVersionInfo myVer =
            System.Diagnostics.FileVersionInfo.GetVersionInfo
            (
                System.Reflection.Assembly.GetExecutingAssembly().Location
            );
        //mail
        private readonly string myMail = "sy7w-husy@asahi-net.or.jp";

        //再生記録ファイル(自身のファイル名.log)
        private readonly string lastfileLog = 
            Path.GetFileNameWithoutExtension(Assembly.GetExecutingAssembly().Location) + ".log";

        //設定ファイル初期値
        private int AllowsTransparency_FLG = 1;//ウィンドウ枠非表示(0:しない,1:する)
        private int strech_FLG = 2;// 0:None,1:Fill,2:Uniform,3:UniformToFill
        private int continue_FLG = 1;//New / Continue 選択画面の表示有無
        private int init_display_delay = 0;//ウェイト時間
        private double init_visibility = 0.3;//コントロールボタンの透過率
        private int topmost_FLG = 1;//最前面表示 1:最前面表示する 0:しない
        private int loop_FLG = 2;//0:終了,1;同ループ,2:次ファイル(名前順),3:次ファイル(作成日順)"
        private double init_display_size = 1.0;        //画面の表示サイズ
        private double init_sliderVolume_Value = 50;        //再生音量
        private string libVlcDllsPath = @"C:\Program Files (x86)\VideoLAN\VLC";//VLCファルダ
        private string libVlcPluginsPath = @"C:\Program Files (x86)\VideoLAN\VLC\plugins";//プラグインファルダ

        //ファイル用文字変数
        private string IniDirectory = "";//ファイルオープンダイアログの初期表示ディレクトリ名
        private string playingFileName = "";//再生ファイル名（フルパス）
        private string PlayingSafeFileName = "";//再生ファイル名（ファイル名のみ）

        //処理用変数
        private bool NewFileOpen = true;//新しくファイルを開く
        private double continuePosition = 0d;//再生ポジション
        private bool myPositionChanging;//再生位置の変更
        private bool ControlBotton_Visibility = false;//コントロールボタンの表示

        //エラーコード
        private int err_CD = 0;
        private int read_FLG = 0;

        //メイン処理
        public MainWindow()
        {
            int lastfileLogExstFL= 0;

            //引数処理、設定ファイル読み込み、New / Continue 選択画面の表示
            Directory.SetCurrentDirectory(App.AppDir);
            InifileLoad();

            //VlcContext設定
            try
            {
                //Set libvlc.dll and libvlccore.dll directory path
                VlcContext.LibVlcDllsPath = libVlcDllsPath;
                //Set the vlc plugins directory path
                VlcContext.LibVlcPluginsPath = libVlcPluginsPath;
                //Set the startup options
                VlcContext.StartupOptions.IgnoreConfig = true;
                VlcContext.StartupOptions.LogOptions.LogInFile = true;
                VlcContext.StartupOptions.LogOptions.ShowLoggerConsole = true;
                //コンソール出力
                //VlcContext.StartupOptions.LogOptions.Verbosity = VlcLogVerbosities.Debug;
                VlcContext.StartupOptions.AddOption("--no-video-title-show");
                //Initialize the VlcContext
                VlcContext.Initialize();
            }
            catch (Exception ex)
            {
                MessageBox.Show(
                    "Program can't run. Check DllsPath," + "PluginsPath and rerun.",
                    "Check Dllpath and PluginsPath",
                    MessageBoxButton.OK,
                    MessageBoxImage.Error
                );
                System.Console.WriteLine(ex.Message);
                err_CD = 1;
                Form_settings form_Settings = new Form_settings();
                form_Settings.ShowDialog();
                Environment.Exit(0);
            }

            //ログファイル読み込み
            if (File.Exists(lastfileLog))
            {
                LastfileLogLoad();
                lastfileLogExstFL = 1;
            }
            //引数がある場合(エクスプローラーからプログラムで開く場合)
            if (App.CommandLineArgs.Length != 0)
            {
                playingFileName = App.CommandLineArgs;
                IniDirectory = Path.GetDirectoryName(App.CommandLineArgs);
                NewFileOpen = false;
                continuePosition = 0d;
                init_sliderVolume_Value = Properties.Settings.Default.init_sliderVolume_Value;
            }
            //起動時ファイルオープン設定
            //ファイルオープンダイアログ表示
            else if (Properties.Settings.Default.continue_FLG == 0)
            {
                NewFileOpen = true;
            }
            //New / Continue 選択画面表示
            else if (Properties.Settings.Default.continue_FLG == 1)
            {
                if (lastfileLogExstFL == 1)
                {
                    InitForm();
                }
                else
                {
                     NewFileOpen = true;
                }
            }
            //即再生再開
            else if (Properties.Settings.Default.continue_FLG == 2 && lastfileLogExstFL == 1)
            {
                LastfileLogLoad();
            }
            //ファイルオープンダイアログ表示
            else
            {
                NewFileOpen = true;
            }
            //初期化
            InitializeComponent();
            sliderVolume.Value =init_sliderVolume_Value;
            myVlcControl.VideoProperties.Scale = 1.0f;
            this.Activate();
            Stretch_Mode_Change(strech_FLG);
            if (AllowsTransparency_FLG == 1) AllowsTransparency = true;
            if (topmost_FLG == 1)
            {
                Topmost = true;
            }
            else
            {
                Topmost = false;
            }

            //タイマー処理(画面スリープ抑止)
            System.Windows.Forms.Timer timer = new System.Windows.Forms.Timer();//タイマー生成
            timer.Tick += new EventHandler(OnTick_Timer);//画面スリープ抑止 & 再生状況書き込み
            timer.Interval = 10000;//10秒ごと処理 2017.09.10 再生状況書き込み機能追加
            timer.Start();//タイマー開始

            //イベントハンドラ
            myVlcControl.PositionChanged += VlcControlOnPositionChanged;
            myVlcControl.TimeChanged += VlcControlOnTimeChanged;

            //再生終了時の処理
            myVlcControl.EndReached += MyVlcControl_EndReached;
            myVlcControl.Paused += MyVlcControl_Paused;
            myVlcControl.Playing += MyVlcControl_Playing;
            Closing += MainWindowOnClosing;
            MouseLeftButtonDown += (sender, e) => DragMove();
            ContextMenu.MouseEnter += ContextMenu_MouseEnter;

            //メディアファイルオープン
            if (NewFileOpen)
            {
                Open_File();
            }
            else
            {
                Play_continue(playingFileName, continuePosition, init_sliderVolume_Value);
            }
        }

        //設定ファイル読込
        private void InifileLoad()
        {
            AllowsTransparency_FLG = Properties.Settings.Default.AllowsTransparency_FLG;
            strech_FLG = Properties.Settings.Default.strech_FLG;
            continue_FLG = Properties.Settings.Default.continue_FLG;
            init_display_delay = Properties.Settings.Default.init_display_delay;
            init_visibility = Properties.Settings.Default.init_visibility;
            topmost_FLG = Properties.Settings.Default.topmost_FLG;
            loop_FLG = Properties.Settings.Default.loop_FLG;
            init_display_size = Properties.Settings.Default.init_display_size;
            init_sliderVolume_Value = Properties.Settings.Default.init_sliderVolume_Value;
            libVlcDllsPath = Properties.Settings.Default.libVlcDllsPath;
            libVlcPluginsPath = Properties.Settings.Default.libVlcPluginsPath;
            if (!Directory.Exists(libVlcDllsPath))
            {
                MessageBox.Show
                    (
                        "Error DllsPath not found. Correct  DllsPath.",
                        "Check Dllpath",
                        MessageBoxButton.OK,
                        MessageBoxImage.Error
                    );
                Form_settings form_Settings = new Form_settings();
                form_Settings.ShowDialog();
                InifileLoad();
            }
            if (!Directory.Exists(libVlcPluginsPath))
            {
                MessageBox.Show
                    (
                        "Error PluginsPath not found. Correct  PluginsPath.",
                        "Check PluginsPath",
                        MessageBoxButton.OK,
                        MessageBoxImage.Error
                    );
                Form_settings form_Settings = new Form_settings();
                form_Settings.ShowDialog();
                InifileLoad();
            }
        }

        //lastfileLog読込 2019.08.03 logfile読込の例外処理追加
        private void LastfileLogLoad()
        {
            StreamReader logfile = new StreamReader(lastfileLog, System.Text.Encoding.UTF8);
            try
            {
                string logfileName = logfile.ReadLine();
                string continueFileDirectory = System.IO.Path.GetDirectoryName(logfileName);
                string continueSafeFileName = logfile.ReadLine();
                string strPosition = logfile.ReadLine();
                string strDuration = logfile.ReadLine();
                init_sliderVolume_Value = double.Parse(logfile.ReadLine());

                if (File.Exists(logfileName))
                {
                    playingFileName = logfileName;
                    continuePosition = double.Parse(strPosition);
                    if (continuePosition < 0 || continuePosition > 1)
                        {
                                continuePosition = 0d;
                        }
                    NewFileOpen = false;
                }
                else
                {
                    NewFileOpen = true;
                }
                logfile.Close();
            }
            catch (Exception ex)
            {
                logfile.Close();
                MessageBox.Show
                    (
                        "Logfile isn't correct.\nOpen new file.",
                        "Logfile Error",
                        MessageBoxButton.OK,
                        MessageBoxImage.Error
                    );
                System.Console.WriteLine(ex.Message);
                StreamWriter w_lastfileLog = 
                    new StreamWriter(new FileStream(lastfileLog, FileMode.Create), System.Text.Encoding.UTF8);
                w_lastfileLog.WriteLine("Open new file.");
                w_lastfileLog.Close();
                NewFileOpen = true;
            }
        }

        //初期ダイアログ表示
        private void InitForm()
        {
            Form_init form_init = new Form_init(lastfileLog);
            form_init.ShowDialog();
            if (form_init.exitProgram) Close();
            NewFileOpen = form_init.newfileopen;
            if (Directory.Exists(form_init.continueFileDirectory))
            {
                IniDirectory = form_init.continueFileDirectory;
            }
            else IniDirectory = "";
            if (!form_init.newfileopen)
            {
                playingFileName = form_init.continueFileName;
                continuePosition = form_init.continuePosition;
            }
        }

        //Open File
        private void Open_File()
        {
            if (myVlcControl.Media != null) myVlcControl.Media.ParsedChanged -= MediaOnParsedChanged;
            OpenFileDialog oFD = new OpenFileDialog();
            if (!Directory.Exists(IniDirectory)) IniDirectory = "";
            oFD.InitialDirectory = IniDirectory;
            oFD.Title = "Open Movie file";
            oFD.FileName = "Movie File";
            oFD.Filter =
             "Movie Files |*.mp4;*.m4v;*.wmv;*.webm;*.flv;*.mov;*.avi;*.mkv;*.asf;*.flv;*.qt";// +
           //"|Other Files|*.jpg" +
           //"|All Files|*.*";
            if (oFD.ShowDialog() != true) return;
            playingFileName = oFD.FileName;
            PlayingSafeFileName = oFD.SafeFileName;
            sliderPosition.Value = 0;
            init_sliderVolume_Value = Properties.Settings.Default.init_sliderVolume_Value;
            Play();
        }

        //Play
        private void Play()
        {
            read_FLG = 0;
            if (myVlcControl.Media != null) Init_display_size_calc();
            PlayingSafeFileName = Path.GetFileName(playingFileName);
            IniDirectory = System.IO.Path.GetDirectoryName(playingFileName);
            myVlcControl.Media = new PathMedia(playingFileName);
            myVlcControl.Media.ParsedChanged += MediaOnParsedChanged;
            myVlcControl.Position = (float)sliderPosition.Value;
            myVlcControl.AudioProperties.Volume = Convert.ToInt32(sliderVolume.Value);
            myVlcControl.AudioProperties.Track = int.Parse(AudioTrackChangeButton.Content.ToString());
            myVlcControl.Rate = 1.0f;
            Set_textRate();
            Title = PlayingSafeFileName;
            TitleText.Text = PlayingSafeFileName;
            if (init_display_size == 0) {
                ScreenFull();
            }
            else {
                ChangeWindowSize(init_display_size);
            }
            ControlButtonHiden();
            read_FLG = 1;
            Playlog();
        }

        //Play Continue
        private void Play_continue(string continuFileName, double continuePosition, double volume)
        {
            if (myVlcControl.Media != null) myVlcControl.Media.ParsedChanged -= MediaOnParsedChanged;
            playingFileName = continuFileName;
            sliderPosition.Value = continuePosition;
            sliderVolume.Value = volume;
            Play();
        }

        //Replay
        private void Replay()
        {
            if (myVlcControl.Media == null) return;
            sliderPosition.Value = 0;
            Play();
        }

        //Play Next
        private void Play_next()
        {
            string dir = Path.GetDirectoryName(playingFileName);
            string ext = Path.GetExtension(playingFileName);
            ext = "*" + ext;
            string[] files = Directory.GetFiles(dir, ext);
            int i = files.Length;
            //名前順に並び替え
            if (loop_FLG == 2) Array.Sort(files);
            //更新日順に並び変え
            if (loop_FLG == 3) Array.Sort(files, CompareLastWriteTime);
            for (int j = 0; j <= i - 1; j++)
            {
                if (files[j] == playingFileName)
                {
                    if (j == i - 1)
                    {
                        playingFileName = files[0];
                        return;
                    }
                    else
                        playingFileName = files[j + 1];
                    return;
                }
            }
        }

        //更新日順に並び変え
        private int CompareLastWriteTime(string fileX, string fileY)
        {
            return DateTime.Compare(File.GetLastWriteTime(fileX), File.GetLastWriteTime(fileY));
        }

        //Play-Pause
        private void MyVlcControl_Playing(VlcControl sender, VlcEventArgs<EventArgs> e)
        {
            PlayButton.Content = "Pause";
        }

        //Pause-Play
        private void MyVlcControl_Paused(VlcControl sender, VlcEventArgs<EventArgs> e)
        {
            PlayButton.Content = "Play";
        }

        //再生終了時の処理
        private void MyVlcControl_EndReached(VlcControl sender, VlcEventArgs<EventArgs> e)
        {
            Init_display_size_calc();
            sliderPosition.Value = 0;
            if (loop_FLG == 0)
            {
                Application.Current.Shutdown();
            }
            else if (loop_FLG == 1)
            {
                Replay();
            }
            else if (loop_FLG == 2 || loop_FLG == 3)
            {
                Play_next();
                Play();
            }
        }

        //Close
        private void MainWindowOnClosing(object sender, CancelEventArgs e)
        {
            Playlog();
            Inifile_write();
            VlcContext.CloseAll();
        }

        //再生状況書き込み
        private void Playlog()
        {
          if (myVlcControl.Media != null && err_CD == 0 && read_FLG == 1)
          {
             Lastfilelog_write();
             Init_display_size_calc();
          }
        }

        //lastfilelog 書き込み
        private void Lastfilelog_write()
        {
            StreamWriter w_lastfileLog =
                new StreamWriter(new FileStream(lastfileLog, FileMode.Create), System.Text.Encoding.UTF8);
            w_lastfileLog.WriteLine(playingFileName);//1
            w_lastfileLog.WriteLine(PlayingSafeFileName);//2
            w_lastfileLog.WriteLine(myVlcControl.Position.ToString());//3
            w_lastfileLog.WriteLine(myVlcControl.Media.Duration.ToString());//4
            w_lastfileLog.WriteLine(init_sliderVolume_Value.ToString());//5
            w_lastfileLog.Close();
           //MessageBox.Show("W Log");
        }

        //設定ファイル書き込み
        private void Inifile_write()
        {
            Properties.Settings.Default.AllowsTransparency_FLG = AllowsTransparency_FLG;
            Properties.Settings.Default.strech_FLG = strech_FLG;
            Properties.Settings.Default.continue_FLG = continue_FLG;
            Properties.Settings.Default.init_display_delay = init_display_delay;
            Properties.Settings.Default.init_visibility = init_visibility;
            Properties.Settings.Default.topmost_FLG = topmost_FLG;
            Properties.Settings.Default.loop_FLG = loop_FLG;
            Properties.Settings.Default.init_display_size = init_display_size;
          //Properties.Settings.Default.init_sliderVolume_Value = init_sliderVolume_Value;
            Properties.Settings.Default.libVlcDllsPath = libVlcDllsPath;
            Properties.Settings.Default.libVlcPluginsPath = libVlcPluginsPath;
            Properties.Settings.Default.Save();
        }

        //オープンボタン
        private void Button_Click(object sender, RoutedEventArgs e)
        {
            Open_File();
        }

        //クローズボタン
        private void CloseButton_Click(object sender, RoutedEventArgs e)
        {
            Application.Current.Shutdown();
        }

        //マウス左クリック
        private void Window_MouseLeftButtonDown(object sender, MouseButtonEventArgs e)
        {
            if (WindowState == WindowState.Maximized)
            {
                if (ControlBotton_Visibility == false) { ControlButtonVisible(); }
                else { ControlButtonHiden(); }
            }
        }

        //ウインドウスタイル変更
        private void WindowStyle_Change()
        {
            if (AllowsTransparency_FLG == 1) return;
            if (WindowStyle == WindowStyle.None)
            {
                WindowStyle = WindowStyle.SingleBorderWindow;
            }
            else
            {
                WindowStyle = WindowStyle.None;
            }
        }

        //Full Screen
        private void ScreenFull()
        {
            WindowStyle = WindowStyle.None;
            WindowState = WindowState.Maximized;
            init_display_size = 0;
            ControlButtonHiden();
            Button_layout(1);
            this.Activate();
        }
        private void ScreenNormal()
        {
            WindowState = WindowState.Normal;
            WindowStyle = WindowStyle.None;
            Init_display_size_calc();
        }
        private void FullScreen()
        {
            if (WindowState != WindowState.Maximized || WindowStyle != WindowStyle.None)
            {
                ScreenFull();
            }
            else
            {
                ScreenNormal();
            }
        }

        //再生スライダー(手動変更)
        private void Slider_ValueChanged(object sender, RoutedPropertyChangedEventArgs<double> e)
        {
            if (myPositionChanging) myVlcControl.Position = (float)e.NewValue;
            //Update the current position text when it is in pause
            TimeSpan duration =  myVlcControl.Media == null ? TimeSpan.Zero : myVlcControl.Media.Duration;
            /*上の式は、次の"if ～ else"構文と同一

            TimeSpan duration = new TimeSpan();
            if (myVlcControl.Media == null)
                duration = TimeSpan.Zero;
            else
                duration = myVlcControl.Media.Duration;

            */
            TimeSpan time = TimeSpan.FromMilliseconds(duration.TotalMilliseconds * myVlcControl.Position);
            textBlock.Text = string.Format
                (
                    "{0:00}:{1:00}:{2:00} / {3:00}:{4:00}:{5:00}",
                    time.Hours,
                    time.Minutes,
                    time.Seconds,
                    duration.Hours,
                    duration.Minutes,
                    duration.Seconds
                );
        }

        //再生スライダー(時間経過)
        private void VlcControlOnTimeChanged(VlcControl sender, VlcEventArgs<TimeSpan> e)
        {
            if (myVlcControl.Media == null) return;
            TimeSpan duration = myVlcControl.Media.Duration;
            textBlock.Text = string.Format
                (
                 "{0:00}:{1:00}:{2:00} / {3:00}:{4:00}:{5:00}"
                 , e.Data.Hours
                 , e.Data.Minutes
                 , e.Data.Seconds
                 , duration.Hours
                 , duration.Minutes
                 , duration.Seconds
                );
            sliderPosition.Value = myVlcControl.Position;// Add 2019/02/16
        }

        //メディアチェンジ
        private void MediaOnParsedChanged(MediaBase sender, VlcEventArgs<int> e)
        {
            textBlock.Text = string.Format
                (
                 "Duration: {0:00}:{1:00}:{2:00}"
                 , myVlcControl.Media.Duration.Hours
                 , myVlcControl.Media.Duration.Minutes
                 , myVlcControl.Media.Duration.Seconds
                );
            sliderPosition.Value = myVlcControl.Position;// Add 2019/02/16
            PlayButton.Content = "Pause";
            sliderVolume.Value = init_sliderVolume_Value;
            myVlcControl.AudioProperties.Volume = Convert.ToInt32(sliderVolume.Value);
            myVlcControl.Rate = 1.0f;
            Set_textRate();
            AudioTrackChangeButton.Content = 1;
        }

        //再生スライダー移動
        private void SliderPosition_PreviewMouseLeftButtonDown(object sender, MouseButtonEventArgs e)
        {
            myPositionChanging = true;
            myVlcControl.PositionChanged -= VlcControlOnPositionChanged;
        }
        private void SliderPosition_PreviewMouseLeftButtonUp(object sender, MouseButtonEventArgs e)
        {
            myVlcControl.Position = (float)sliderPosition.Value;
            myVlcControl.PositionChanged += VlcControlOnPositionChanged;
            myPositionChanging = false;
        }

        //再生スライダー移動 Rewind/Forward Button
        private void Button_Pos_R_Click(object sender, RoutedEventArgs e)
        {
            sliderPosition.Value -= 0.00025;
            myVlcControl.Position = (float)sliderPosition.Value;
            myVlcControl.PositionChanged -= VlcControlOnPositionChanged;
            myPositionChanging = false;
        }
        private void Button_Pos_F_Click(object sender, RoutedEventArgs e)
        {
            sliderPosition.Value += 0.00025;
            myVlcControl.Position = (float)sliderPosition.Value;
            myVlcControl.PositionChanged -= VlcControlOnPositionChanged;
            myPositionChanging = false;
        }

        //Play/Pause
        private void Play_Click(object sender, RoutedEventArgs e)
        {
            Play_pause();
        }
        //Play/Pause
        private void Play_pause()
        {
            if (myVlcControl.Media != null) myVlcControl.Pause();
        }

        //ボリューム初期値
        private void SliderVolume_Initialized(object sender, EventArgs e)
        {
            sliderVolume.Value = init_sliderVolume_Value;
        }

        //ボリュームスライダー位置セット
        private void VlcControlOnPositionChanged(VlcControl sender, VlcEventArgs<float> e)
        {
            if (myPositionChanging)
            {
                // User is currently changing the position using the slider, so do not update.
                return;
            }
            sliderPosition.Value = e.Data;
        }

        //ボリューム変更(Slider）
        private void SliderVolume_ValueChanged(object sender, RoutedPropertyChangedEventArgs<double> e)
        {
            myVlcControl.AudioProperties.Volume = Convert.ToInt32(sliderVolume.Value);
            init_sliderVolume_Value = sliderVolume.Value;
        }

        //ボリューム変更(MouseWheel)
        private void CuteMP4_MouseWheel(object sender, MouseWheelEventArgs e)
        {
            if (e.Delta > 0)
            {
                sliderVolume.Value += 5;
            }
            if (e.Delta < 0)
            {
                sliderVolume.Value -= 5;
            }
            init_sliderVolume_Value = sliderVolume.Value;
        }

        //ボリューム変更(Up/Down Button)
        private void Button_Vol_UP_Click(object sender, RoutedEventArgs e)
        {
            sliderVolume.Value += 0.25;
            init_sliderVolume_Value = sliderVolume.Value;
        }
        private void Button_Vol_Down_Click(object sender, RoutedEventArgs e)
        {
            sliderVolume.Value -= 0.25;
            init_sliderVolume_Value = sliderVolume.Value;
        }

        //ボリューム変更(Mute)
        private void CheckboxMute_Checked(object sender, RoutedEventArgs e)
        {
            myVlcControl.AudioProperties.IsMute = true;
        }
        private void CheckboxMute_Unchecked(object sender, RoutedEventArgs e)
        {
            myVlcControl.AudioProperties.IsMute = false;
        }

        //オーディオトラック変更
        private void AudioTrackChangeButton_Click(object sender, RoutedEventArgs e)
        {
            AudioTrackChange();
        }
        private void AudioTrackChange()
        {
            int AudioTrackCount = 1;
            int AudioTrack = 1;
            if (myVlcControl.Media != null)
            {
                AudioTrackCount = myVlcControl.AudioProperties.TrackCount;
                AudioTrack = myVlcControl.AudioProperties.Track;
                AudioTrack += 1;
                if (AudioTrack >= AudioTrackCount) AudioTrack = 1;
                myVlcControl.AudioProperties.Track = AudioTrack;
            }
            AudioTrackChangeButton.Content = AudioTrack;
        }

        //Stretchモード変更
        private void Stretch_Mode_Change(int i)
        {
            strech_FLG = i;
            switch (i)
            {
                case 0:
                    Image_Media.Stretch = System.Windows.Media.Stretch.None;
                    break;
                case 1:
                    Image_Media.Stretch = System.Windows.Media.Stretch.Fill;
                    break;
                case 2:
                    Image_Media.Stretch = System.Windows.Media.Stretch.Uniform;
                    break;
                case 3:
                    Image_Media.Stretch = System.Windows.Media.Stretch.UniformToFill;
                    break;
            }
        }

        //ボタンレイアウト変更
        private void Button_layout(int i)
        {
            switch (i)
            {
                case 1://フルスクリーン時
                    OpenButton.Width = 160;
                    OpenButton.Height = 100;
                    CloseButton.Width = 160;
                    CloseButton.Height = 100;
                    PlayButton.Margin = new Thickness(0, 100, 0, 0);
                    PlayButton.Width = 160;
                    PlayButton.Height = 100;
                    textRate.Margin = new Thickness(0, 100, 30, 0);
                    textRate.Width = 100;
                    textRate.Height = 50;
                    AudioTrackChangeButton.Margin = new Thickness(0, 0, 86, 35);
                    AudioTrackChangeButton.Width = 50;
                    AudioTrackChangeButton.Height = 50;
                    checkboxMute.Width = 50;
                    checkboxMute.Height = 50;
                    Button_Vol_UP.Height = 150;
                    Button_Vol_Down.Height = 150;
                    sliderVolume.Margin = new Thickness(0, 150, 0, 185);
                    Button_Pos_R.Width = 150;
                    Button_Pos_F.Width = 150;
                    sliderPosition.Margin = new Thickness(150, 0, 185, 0);
                    break;
                case 2://フルスクリーン以外
                    OpenButton.Width = 80;
                    OpenButton.Height = 25;
                    CloseButton.Width = 80;
                    CloseButton.Height = 25;
                    PlayButton.Margin = new Thickness(0, 25, 0, 0);
                    PlayButton.Width = 80;
                    PlayButton.Height = 25;
                    textRate.Margin = new Thickness(0, 25, 30, 0);
                    textRate.Width = 50;
                    textRate.Height = 25;
                    AudioTrackChangeButton.Margin = new Thickness(0, 0, 51, 35);
                    AudioTrackChangeButton.Width = 15;
                    AudioTrackChangeButton.Height = 15;
                    checkboxMute.Width = 15;
                    checkboxMute.Height = 15;
                    Button_Vol_UP.Height = 30;
                    Button_Vol_Down.Height = 30;
                    sliderVolume.Margin = new Thickness(0, 30, 0, 65);
                    Button_Pos_R.Width = 50;
                    Button_Pos_F.Width = 50;
                    sliderPosition.Margin = new Thickness(50, 0, 85, 0);
                    break;
            }
        }

        //コントロールボタン表示
        private void ControlButtonVisible()
        {
            this.Activate();
            //this.Activate();2018/01/20  Window_MouseEntへ移動
            if (Height > 90 && Width > 120) //2017.12.03 ADD ウィンドウサイズが小さいときボタン非表示
            {
                OpenButton.Visibility = Visibility.Visible;
                OpenButton.Opacity = init_visibility;
                PlayButton.Visibility = Visibility.Visible;
                PlayButton.Opacity = init_visibility;
                CloseButton.Visibility = Visibility.Visible;
                CloseButton.Opacity = init_visibility;
                sliderVolume.Visibility = Visibility.Visible;
                sliderVolume.Opacity = init_visibility;
                AudioTrackChangeButton.Visibility = Visibility.Visible;
                AudioTrackChangeButton.Opacity = init_visibility;
                checkboxMute.Visibility = Visibility.Visible;
                checkboxMute.Opacity = init_visibility;
                TitleText.Visibility = Visibility.Visible;
                TitleText.Opacity = init_visibility;
                textBlock.Visibility = Visibility.Visible;
                textBlock.Opacity = init_visibility;
                textRate.Visibility = Visibility.Visible;
                textRate.Opacity = init_visibility;
                sliderPosition.Value = myVlcControl.Position;// Add 2019/02/16
                sliderPosition.Visibility = Visibility.Visible;
                sliderPosition.Opacity = init_visibility;
                CheckBoxGrip.Visibility = Visibility.Visible;
                CheckBoxGrip.Opacity = init_visibility;
                Button_Vol_UP.Visibility = Visibility.Visible;
                Button_Vol_UP.Opacity = init_visibility;
                Button_Vol_Down.Visibility = Visibility.Visible;
                Button_Vol_Down.Opacity = init_visibility;
                Button_Pos_R.Visibility = Visibility.Visible;
                Button_Pos_R.Opacity = init_visibility;
                Button_Pos_F.Visibility = Visibility.Visible;
                Button_Pos_F.Opacity = init_visibility;
            }
            Mouse.OverrideCursor = Cursors.Cross;
            ControlBotton_Visibility = true;
        }

        //コントロールボタン非表示
        private void ControlButtonHiden()
        {
            this.Activate();
            OpenButton.Visibility = Visibility.Hidden;
            PlayButton.Visibility = Visibility.Hidden;
            CloseButton.Visibility = Visibility.Hidden;
            sliderVolume.Visibility = Visibility.Hidden;
            AudioTrackChangeButton.Visibility = Visibility.Hidden;
            checkboxMute.Visibility = Visibility.Hidden;
            TitleText.Visibility = Visibility.Hidden;
            textBlock.Visibility = Visibility.Hidden;
            textRate.Visibility = Visibility.Hidden;
            sliderPosition.Visibility = Visibility.Hidden;
            CheckBoxGrip.Visibility = Visibility.Hidden;
            Button_Vol_UP.Visibility = Visibility.Hidden;
            Button_Vol_Down.Visibility = Visibility.Hidden;
            Button_Pos_R.Visibility = Visibility.Hidden;
            Button_Pos_F.Visibility = Visibility.Hidden;
            Mouse.OverrideCursor = Cursors.None;
            ControlBotton_Visibility = false;
        }

        //Cursor Visibility
        private void Window_MouseEnter(object sender, MouseEventArgs e)
        {
            this.Activate();
            if (WindowState != WindowState.Maximized || WindowStyle != WindowStyle.None)
            {
                ControlButtonVisible();
             }
            else
            {
                ControlButtonHiden();
            }
        }
        private void CuteMP4_MouseLeave(object sender, MouseEventArgs e)
        {
            ControlButtonHiden();
        }
        private void OpenButton_MouseEnter(object sender, MouseEventArgs e)
        {
            Mouse.OverrideCursor = Cursors.Hand;
            OpenButton.Opacity = 1;
            OpenButton.ToolTip = "Open media file.'O'";
        }
        private void OpenButton_MouseLeave(object sender, MouseEventArgs e)
        {
            Mouse.OverrideCursor = Cursors.Cross;
            OpenButton.Opacity = init_visibility;
        }
        private void PlayButton_MouseEnter(object sender, MouseEventArgs e)
        {
            Mouse.OverrideCursor = Cursors.Hand;
            PlayButton.Opacity = 1;
            PlayButton.ToolTip = "Play or Pause.'P','Space'";
        }
        private void PlayButton_MouseLeave(object sender, MouseEventArgs e)
        {
            Mouse.OverrideCursor = Cursors.Cross;
            PlayButton.Opacity = init_visibility;
        }
        private void CloseButton_MouseEnter(object sender, MouseEventArgs e)
        {
            Mouse.OverrideCursor = Cursors.Hand;
            CloseButton.Opacity = 1;
            CloseButton.ToolTip = "Close file and Exit program.'C'";
        }
        private void CloseButton_MouseLeave(object sender, MouseEventArgs e)
        {
            Mouse.OverrideCursor = Cursors.Cross;
            CloseButton.Opacity = init_visibility;
        }
        private void SliderVolume_MouseEnter(object sender, MouseEventArgs e)
        {
            Mouse.OverrideCursor = Cursors.Hand;
            sliderVolume.Opacity = 1;
            sliderVolume.ToolTip = "Volume Up/Down.'UP','PageUp','Down','PageDown','MouseWheel'";
        }
        private void SliderVolume_MouseLeave(object sender, MouseEventArgs e)
        {
            Mouse.OverrideCursor = Cursors.Cross;
            sliderVolume.Opacity = init_visibility;
        }
        private void CheckboxMute_MouseEnter(object sender, MouseEventArgs e)
        {
            Mouse.OverrideCursor = Cursors.Hand;
            checkboxMute.Opacity = 1;
            //ToolTipが左下に表示される場合、最後の'.の表示位置が正しく表示されない
            // exp. checkboxMute.ToolTip = "Mute.'M'"; では正しく表示されない
            checkboxMute.ToolTip = "'Mute.'M";
        }
        private void CheckboxMute_MouseLeave(object sender, MouseEventArgs e)
        {
            Mouse.OverrideCursor = Cursors.Cross;
            checkboxMute.Opacity = init_visibility;
        }
        private void SliderPosition_MouseEnter(object sender, MouseEventArgs e)
        {
            Mouse.OverrideCursor = Cursors.Hand;
            sliderPosition.Opacity = 1;
            textBlock.Opacity = 1;
            sliderPosition.ToolTip = "Playing position.'Left','Right'";
        }
        private void SliderPosition_MouseLeave(object sender, MouseEventArgs e)
        {
            Mouse.OverrideCursor = Cursors.Cross;
            sliderPosition.Opacity = init_visibility;
            textBlock.Opacity = init_visibility;
        }
        private void AudioTrackChangeButton_MouseEnter(object sender, MouseEventArgs e)
        {
            Mouse.OverrideCursor = Cursors.Hand;
            AudioTrackChangeButton.Opacity = 1;
            AudioTrackChangeButton.ToolTip = "AudioTrack.'A'";
        }
        private void AudioTrackChangeButton_MouseLeave(object sender, MouseEventArgs e)
        {
            Mouse.OverrideCursor = Cursors.Cross;
            AudioTrackChangeButton.Opacity = init_visibility;
        }
        private void TitleText_MouseEnter(object sender, MouseEventArgs e)
        {
            TitleText.Opacity = 1;
            TitleText.ToolTip = PlayingSafeFileName;
        }
        private void TitleText_MouseLeave(object sender, MouseEventArgs e)
        {
            TitleText.Opacity = init_visibility;
        }
        private void TextBlock_MouseEnter(object sender, MouseEventArgs e)
        {
            textBlock.Opacity = 1;
            textBlock.ToolTip = "Playing Position / Media Duration. ";
        }
        private void TextBlock_MouseLeave(object sender, MouseEventArgs e)
        {
            textBlock.Opacity = init_visibility;
        }
        private void TextRate_MouseEnter(object sender, MouseEventArgs e)
        {
            Mouse.OverrideCursor = Cursors.Hand;
            textRate.Opacity = 1;
            textRate.ToolTip = "Play Speed Rate.'1','2','3','4','5','6','Q',W','E','R','T'";
        }
        private void TextRate_MouseLeave(object sender, MouseEventArgs e)
        {
            Mouse.OverrideCursor = Cursors.Cross;
            textRate.Opacity = init_visibility;
        }
        private void CheckBoxGrip_MouseEnter(object sender, MouseEventArgs e)
        {
            Mouse.OverrideCursor = Cursors.Hand;
            CheckBoxGrip.Opacity = 1;
            CheckBoxGrip.ToolTip = "Risize Grip.'V'";
        }
        private void CheckBoxGrip_MouseLeave(object sender, MouseEventArgs e)
        {
            Mouse.OverrideCursor = Cursors.Cross;
            CheckBoxGrip.Opacity = init_visibility;
        }
        private void Button_Vol_UP_MouseEnter(object sender, MouseEventArgs e)
        {
            Mouse.OverrideCursor = Cursors.Hand;
            Button_Vol_UP.Opacity = 1;
            Button_Vol_UP.ToolTip = "Volume Up.'Up','PageUp','MouseWheel'";
        }
        private void Button_Vol_UP_MouseLeave(object sender, MouseEventArgs e)
        {
            Mouse.OverrideCursor = Cursors.Cross;
            Button_Vol_UP.Opacity = init_visibility;
        }
        private void Button_Vol_Down_MouseEnter(object sender, MouseEventArgs e)
        {
            Mouse.OverrideCursor = Cursors.Hand;
            Button_Vol_Down.Opacity = 1;
            Button_Vol_Down.ToolTip = "Volume Down.'Down','PageDown','MouseWheel'";
        }
        private void Button_Vol_Down_MouseLeave(object sender, MouseEventArgs e)
        {
            Mouse.OverrideCursor = Cursors.Cross;
            Button_Vol_Down.Opacity = init_visibility;
        }
        private void Button_Pos_R_MouseEnter(object sender, MouseEventArgs e)
        {
            Mouse.OverrideCursor = Cursors.Hand;
            Button_Pos_R.Opacity = 1;
            Button_Pos_R.ToolTip = "Play Position Rewind.'Left'";
        }
        private void Button_Pos_R_MouseLeave(object sender, MouseEventArgs e)
        {
            Mouse.OverrideCursor = Cursors.Cross;
            Button_Pos_R.Opacity = init_visibility;
        }
        private void Button_Pos_F_MouseEnter(object sender, MouseEventArgs e)
        {
            Mouse.OverrideCursor = Cursors.Hand;
            Button_Pos_F.Opacity = 1;
            Button_Pos_F.ToolTip = "Play Position Forward.'Right'";
        }
        private void Button_Pos_F_MouseLeave(object sender, MouseEventArgs e)
        {
            Mouse.OverrideCursor = Cursors.Cross;
            Button_Pos_F.Opacity = init_visibility;
        }
        private void ContextMenu_MouseEnter(object sender, MouseEventArgs e)
        {
            Mouse.OverrideCursor = Cursors.Hand;
        }
        //ダブルクリック ※private void ControlBotton_Display()が1回処理された後に実行される
        private void CuteMP4_MouseDoubleClick(object sender, MouseButtonEventArgs e)
        {
            if (WindowState != WindowState.Maximized || WindowStyle != WindowStyle.None)
            {
                ScreenFull();
                //※private void ControlBotton_Display()が1回処理されている
                ControlButtonVisible();
            }
            else
            {
                WindowState = WindowState.Normal;
                WindowStyle = WindowStyle.None;
                init_display_size = 1;
                Init_display_size_calc();
                //※private void ControlBotton_Display()が1回処理されている
                ControlButtonVisible();
            }
        }

        //KEY入力処理
        private void CuteMP4_KeyDown(object sender, KeyEventArgs e)
        {
            if ((Keyboard.GetKeyStates(Key.Space) & KeyStates.Down) > 0)
            {
                Play_pause();
                return;
            }
            if ((Keyboard.GetKeyStates(Key.Enter) & KeyStates.Down) > 0)
            {
                FullScreen();
                return;
            }
            if ((Keyboard.GetKeyStates(Key.Up) & KeyStates.Down) > 0)
            {
                sliderVolume.Value += 0.5;
                init_sliderVolume_Value = sliderVolume.Value;
                return;
            }
            if ((Keyboard.GetKeyStates(Key.Down) & KeyStates.Down) > 0)
            {
                sliderVolume.Value -= 0.5;
                init_sliderVolume_Value = sliderVolume.Value;
                return;
            }
            if ((Keyboard.GetKeyStates(Key.PageUp) & KeyStates.Down) > 0)
            {
                sliderVolume.Value += 0.5;
                init_sliderVolume_Value = sliderVolume.Value;
                return;
            }
            if ((Keyboard.GetKeyStates(Key.PageDown) & KeyStates.Down) > 0)
            {
                sliderVolume.Value -= 0.5;
                init_sliderVolume_Value = sliderVolume.Value;
                return;
            }
            if ((Keyboard.GetKeyStates(Key.D1) & KeyStates.Down) > 0)
            {
                myVlcControl.Rate = 1.0f;
                Set_textRate();
                return;
            }
            if ((Keyboard.GetKeyStates(Key.D2) & KeyStates.Down) > 0)
            {
                myVlcControl.Rate = 2.0f;
                Set_textRate();
                return;
            }
            if ((Keyboard.GetKeyStates(Key.D3) & KeyStates.Down) > 0)
            {
                myVlcControl.Rate = 4.0f;
                Set_textRate();
                return;
            }
            if ((Keyboard.GetKeyStates(Key.D4) & KeyStates.Down) > 0)
            {
                myVlcControl.Rate = 8.0f;
                Set_textRate();
                return;
            }
            if ((Keyboard.GetKeyStates(Key.D5) & KeyStates.Down) > 0)
            {
                myVlcControl.Rate = 16.0f;
                Set_textRate();
                return;
            }
            if ((Keyboard.GetKeyStates(Key.D6) & KeyStates.Down) > 0)
            {
                myVlcControl.Rate = 32.0f;
                Set_textRate();
                return;
            }
            if ((Keyboard.GetKeyStates(Key.D7) & KeyStates.Down) > 0)
            {
                ChangeWindowSize(1.0d);
                return;
            }
            if ((Keyboard.GetKeyStates(Key.D8) & KeyStates.Down) > 0)
            {
                ChangeWindowSize(0.5d);
                return;
            }
            if ((Keyboard.GetKeyStates(Key.D9) & KeyStates.Down) > 0)
            {
                ChangeWindowSize(0.3d);
                return;
            }
            if ((Keyboard.GetKeyStates(Key.D0) & KeyStates.Down) > 0)
            {
                ChangeWindowSize(0.2d);
                return;
            }
            if ((Keyboard.GetKeyStates(Key.OemMinus) & KeyStates.Down) > 0)
            {
                Input_init_display_size();
                return;
            }
            if ((Keyboard.GetKeyStates(Key.Q) & KeyStates.Down) > 0)
            {
                myVlcControl.Rate = 1.0f / 2.0f;
                Set_textRate();
                return;
            }
            if ((Keyboard.GetKeyStates(Key.W) & KeyStates.Down) > 0)
            {
                myVlcControl.Rate = 1.0f / 4.0f;
                Set_textRate();
                return;
            }
            if ((Keyboard.GetKeyStates(Key.E) & KeyStates.Down) > 0)
            {
                myVlcControl.Rate = 1.0f / 8.0f;
                Set_textRate();
                return;
            }
            if ((Keyboard.GetKeyStates(Key.R) & KeyStates.Down) > 0)
            {
                myVlcControl.Rate = 1.0f / 16.0f;
                Set_textRate();
                return;
            }
            if ((Keyboard.GetKeyStates(Key.T) & KeyStates.Down) > 0)
            {
                myVlcControl.Rate = 1.0f / 32.0f;
                Set_textRate();
                return;
            }
            if ((Keyboard.GetKeyStates(Key.O) & KeyStates.Down) > 0)
            {
                Open_File();
                return;
            }
            if ((Keyboard.GetKeyStates(Key.P) & KeyStates.Down) > 0)
            {
                Play_pause();
                return;
            }
            if ((Keyboard.GetKeyStates(Key.A) & KeyStates.Down) > 0)
            {
                AudioTrackChange();
                return;
            }
            if ((Keyboard.GetKeyStates(Key.S) & KeyStates.Down) > 0)
            {
                Replay();
                return;
            }
            if ((Keyboard.GetKeyStates(Key.D) & KeyStates.Down) > 0)
            {
                if (ControlBotton_Visibility == false)
                {
                    ControlButtonVisible();
                }
                else
                {
                    ControlButtonHiden();
                }
                return;
            }
            if ((Keyboard.GetKeyStates(Key.F) & KeyStates.Down) > 0)
            {
                FullScreen();
                return;
            }
            if ((Keyboard.GetKeyStates(Key.G) & KeyStates.Down) > 0)
            {
                WindowStyle_Change();
                return;
            }
            if ((Keyboard.GetKeyStates(Key.C) & KeyStates.Down) > 0)
            {
                Application.Current.Shutdown();
                return;
            }
            if ((Keyboard.GetKeyStates(Key.M) & KeyStates.Down) > 0)
            {
                checkboxMute.IsChecked = !checkboxMute.IsChecked;
                return;
            }
            if ((Keyboard.GetKeyStates(Key.V) & KeyStates.Down) > 0)
            {
                CheckBoxGrip.IsChecked = !CheckBoxGrip.IsChecked;
                return;
            }
            if ((Keyboard.GetKeyStates(Key.Left) & KeyStates.Down) > 0)
            {
                sliderPosition.Value -= 0.001;
                myVlcControl.Position = (float)sliderPosition.Value;
                myVlcControl.PositionChanged -= VlcControlOnPositionChanged;
                myPositionChanging = false;
                return;
            }
            if ((Keyboard.GetKeyStates(Key.Right) & KeyStates.Down) > 0)
            {
                sliderPosition.Value += 0.001;
                myVlcControl.Position = (float)sliderPosition.Value;
                myVlcControl.PositionChanged -= VlcControlOnPositionChanged;
                myPositionChanging = false;
                return;
            }
            if ((Keyboard.GetKeyStates(Key.OemTilde) & KeyStates.Down) > 0)//@ Key
            {
                strech_FLG++;
                if (strech_FLG == 4) strech_FLG = 0;
                Stretch_Mode_Change(strech_FLG);
                return;
            }
        }

        //再生スピード処理
        private void TextRate_Click(object sender, RoutedEventArgs e)
        {
            if (myVlcControl.Rate < 32.0f)
                myVlcControl.Rate *= 2.0f;
            else
                myVlcControl.Rate = 1.0f / 32.0f;
            Set_textRate();
        }
        private void Set_textRate()
        {
            if (myVlcControl.Rate == 1.0f / 32.0f)
            {
                textRate.Content = "x 1/32";
                return;
            }
            if (myVlcControl.Rate == 1.0f / 16.0f)
            {
                textRate.Content = "x 1/16";
                return;
            }
            if (myVlcControl.Rate == 1.0f / 8.0f)
            {
                textRate.Content = "x 1/8";
                return;
            }
            if (myVlcControl.Rate == 1.0f / 4.0f)
            {
                textRate.Content = "x 1/4";
                return;
            }
            if (myVlcControl.Rate == 1.0f / 2.0f)
            {
                textRate.Content = "x 1/2";
                return;
            }
            if (myVlcControl.Rate == 1.0f)
            {
                textRate.Content = "x 1";
                return;
            }
            if (myVlcControl.Rate == 2.0f)
            {
                textRate.Content = "x 2";
                return;
            }
            if (myVlcControl.Rate == 4.0f)
            {
                textRate.Content = "x 4";
                return;
            }
            if (myVlcControl.Rate == 8.0f)
            {
                textRate.Content = "x 8";
                return;
            }
            if (myVlcControl.Rate == 16.0f)
            {
                textRate.Content = "x 16";
                return;
            }
            if (myVlcControl.Rate == 32.0f)
            {
                textRate.Content = "x 32";
                return;
            }
            if (myVlcControl.Rate == 1.0f / 32.0f)
            {
                textRate.Content = "x 1/32";
                return;
            }
        }

        // init_display_size 計算
        private void Init_display_size_calc()
        {
            if (myVlcControl.Media != null)
            {
                if (WindowStyle == WindowStyle.None && WindowState == WindowState.Maximized)
                {
                    init_display_size = 0;
                    Button_layout(1);
                    return;
                }

                System.Threading.Thread.Sleep(init_display_delay);
                try
                {
                    Size videoSize = myVlcControl.VideoProperties.Size;
                    init_display_size = Width / videoSize.Width;
                    Button_layout(2);
                    if (init_display_delay > 10) init_display_delay -= 10;
                }
                catch (Exception ex)
                {
                    init_display_delay += 50;
                    if (init_display_delay >= 5000)
                    {
                        MessageBox.Show
                            (
                                "Delay Over. Quit program.err_2",
                                "Unkown error",
                                MessageBoxButton.OK,
                                MessageBoxImage.Error
                            );
                        System.Console.WriteLine(ex.Message);
                        err_CD = 2;
                        Application.Current.Shutdown();
                    }
                    Init_display_size_calc();
                }

            }
        }
        //コンテキストメニュー


        private void About_this(object sender, RoutedEventArgs e)
        {
            string str;
            str = "Copyright: " + myVer.LegalCopyright.ToString() + "\n";
            str += "Version: " + myVer.FileVersion.ToString() + "\n";
            str += "Update: " + myInfo.LastWriteTime.ToString() + "\n";
            str += "Mail: " + myMail + "\n";
            MessageBox.Show
                (
                    str,
                    "About " + myVer.ProductName.ToString(),
                    MessageBoxButton.OK,
                    MessageBoxImage.Information
                );
        }

        private void MenuItem_fullScreen(object sender, RoutedEventArgs e)
        {
            if (ControlBotton_Visibility == false)
            {
                ControlButtonVisible();
            }
            else
            {
                ControlButtonHiden();
            }
            FullScreen();
        }
        private void MenuItem_Display_Control(object sender, RoutedEventArgs e)
        {
            if (ControlBotton_Visibility == false)
            {
                ControlButtonVisible();
            }
            else
            {
                ControlButtonHiden();
            }
        }
        private void MenuItem_WindowStyle_Change(object sender, RoutedEventArgs e)
        {
            WindowStyle_Change();
        }
        private void MenuItem_Replay(object sender, RoutedEventArgs e)
        {
            Replay();
        }
        private void MenuItem_NormalScreen(object sender, RoutedEventArgs e)
        {
            if (AllowsTransparency_FLG == 1) return;
            WindowStyle = WindowStyle.SingleBorderWindow;
            WindowState = WindowState.Normal;
            init_display_size = 1.0d;
            ChangeWindowSize(init_display_size);
        }
        private void MenuItem_MaxmumScreen(object sender, RoutedEventArgs e)
        {
            if (AllowsTransparency_FLG == 1) return;
            WindowStyle = WindowStyle.SingleBorderWindow;
            WindowState = WindowState.Maximized;
        }
        private void MenuItem_MinimumScreen(object sender, RoutedEventArgs e)
        {
            if (AllowsTransparency_FLG == 1) return;
            WindowStyle = WindowStyle.SingleBorderWindow;
            WindowState = WindowState.Minimized;
        }
        private void MenuItem_Window_Mode(object sender, RoutedEventArgs e)
        {
            if (AllowsTransparency_FLG == 0)
            {
                AllowsTransparency_FLG = 1;
            }
            else
            {
                AllowsTransparency_FLG = 0;
            }
        }
        private void MenuItem_Topmost(object sender, RoutedEventArgs e)
        {
            if (topmost_FLG == 0)
            {
                topmost_FLG = 1;
                Topmost = true;
            }
            else
            {
                topmost_FLG = 0;
                Topmost = false;
            }
        }
        private void MenuItem_Pause(object sender, RoutedEventArgs e)
        {
            Play_pause();
        }
        private void MenuItem_Replaye(object sender, RoutedEventArgs e)
        {
            Replay();
        }
        private void MenuItem_ChangeAudioTrack(object sender, RoutedEventArgs e)
        {
            AudioTrackChange();
        }
        private void MenuItem_Mute(object sender, RoutedEventArgs e)
        {
            checkboxMute.IsChecked = !checkboxMute.IsChecked;
        }
        private void MenuItem_Open(object sender, RoutedEventArgs e)
        {
            Open_File();
        }
        private void MenuItem_Close(object sender, RoutedEventArgs e)
        {
            Application.Current.Shutdown();
        }
        private void MenuItem_rg(object sender, RoutedEventArgs e)
        {
            CheckBoxGrip.IsChecked = !CheckBoxGrip.IsChecked;
        }
        private void MenuItem_m1(object sender, RoutedEventArgs e)
        {
            init_display_size = 1.0d;
            ChangeWindowSize(init_display_size);
        }
        private void MenuItem_S1_2(object sender, RoutedEventArgs e)
        {
            init_display_size = 0.5d;
            ChangeWindowSize(init_display_size);
        }
        private void MenuItem_S1_3(object sender, RoutedEventArgs e)
        {
            init_display_size = 0.3d;
            ChangeWindowSize(init_display_size);
        }
        private void MenuItem_S1_4(object sender, RoutedEventArgs e)
        {
            init_display_size = 0.2d;
            ChangeWindowSize(init_display_size);
        }
        private void MenuItem_SX(object sender, RoutedEventArgs e)
        {
            Input_init_display_size();
        }
        private void MenuItem_Rate_Change_1(object sender, RoutedEventArgs e)
        {
            myVlcControl.Rate = 1.0f;
            Set_textRate();
        }
        private void MenuItem_Rate_Change_2(object sender, RoutedEventArgs e)
        {
            myVlcControl.Rate = 2.0f;
            Set_textRate();
        }
        private void MenuItem_Rate_Change_4(object sender, RoutedEventArgs e)
        {
            myVlcControl.Rate = 4.0f;
            Set_textRate();
        }
        private void MenuItem_Rate_Change_8(object sender, RoutedEventArgs e)
        {
            myVlcControl.Rate = 8.0f;
            Set_textRate();
        }
        private void MenuItem_Rate_Change_16(object sender, RoutedEventArgs e)
        {
            myVlcControl.Rate = 16.0f;
            Set_textRate();
        }
        private void MenuItem_Rate_Change_32(object sender, RoutedEventArgs e)
        {
            myVlcControl.Rate = 32.0f;
            Set_textRate();
        }
        private void MenuItem_Rate_Change_1_2(object sender, RoutedEventArgs e)
        {
            myVlcControl.Rate = 1.0f / 2.0f;
            Set_textRate();
        }
        private void MenuItem_Rate_Change_1_4(object sender, RoutedEventArgs e)
        {
            myVlcControl.Rate = 1.0f / 4.0f;
            Set_textRate();
        }
        private void MenuItem_Rate_Change_1_8(object sender, RoutedEventArgs e)
        {
            myVlcControl.Rate = 1.0f / 8.0f;
            Set_textRate();
        }
        private void MenuItem_Rate_Change_1_16(object sender, RoutedEventArgs e)
        {
            myVlcControl.Rate = 1.0f / 16.0f;
            Set_textRate();
        }
        private void MenuItem_Rate_Change_1_32(object sender, RoutedEventArgs e)
        {
            myVlcControl.Rate = 1.0f / 32.0f;
            Set_textRate();
        }
        private void MenuItem_Quit(object sender, RoutedEventArgs e)
        {
            loop_FLG = 0;
        }
        private void MenuItem_Replay_Loop(object sender, RoutedEventArgs e)
        {
            loop_FLG = 1;
        }
        private void MenuItem_Play_Next_name(object sender, RoutedEventArgs e)
        {
            loop_FLG = 2;
        }
        private void MenuItem_Play_Next_time(object sender, RoutedEventArgs e)
        {
            loop_FLG = 3;
        }
        private void MenuItem_Strecth_None(object sender, RoutedEventArgs e)
        {
            Stretch_Mode_Change(0);
        }
        private void MenuItem_Strecth_Fill(object sender, RoutedEventArgs e)
        {
            Stretch_Mode_Change(1);
        }
        private void MenuItem_Strecth_uniform(object sender, RoutedEventArgs e)
        {
            Stretch_Mode_Change(2);
        }
        private void MenuItem_Strecth_uniformToFill(object sender, RoutedEventArgs e)
        {
            Stretch_Mode_Change(3);
        }
        private void MenuItem_Set_parameters(object sender, RoutedEventArgs e)
        {
            Set_parameters();
        }

        //window サイズ変更
        private void ChangeWindowSize(double multi)
        {
            // if (myVlcControl.Media == null) return;
            if (multi == 0)
            {
                ScreenFull();
                return;
            }
            System.Threading.Thread.Sleep(init_display_delay);
            try
            {
                init_display_size = multi;
                WindowState = WindowState.Normal;
                Size videoSize = myVlcControl.VideoProperties.Size;
                double ah = myVlcControl.Height;
                double aw = myVlcControl.Width;
                double VH = videoSize.Height * init_display_size;
                double VW = videoSize.Width * init_display_size;
                Size VSize = new Size(VH, VW);
                Height = VH;
                Width = VW;
                Button_layout(2);
                if (init_display_delay > 10) init_display_delay -= 10;
            }
            catch (Exception ex)
            {
                init_display_delay += 50;
                if (init_display_delay >= 5000)
                {
                    MessageBox.Show
                        (
                            "Delay Over. Quit program.err_3",
                            "Unkown error",
                            MessageBoxButton.OK,
                            MessageBoxImage.Error
                        );
                    System.Console.WriteLine(ex.Message);
                    err_CD = 3;
                    Application.Current.Shutdown();
                }
                ChangeWindowSize(init_display_size);
            }
        }

        //ウィンドウサイズ変更ダイアログ表示
        private void Input_init_display_size()
        {
            Init_display_size_calc();
            Form_init_display_size display_size = new Form_init_display_size(init_display_size);
            display_size.ShowDialog();
            if (init_display_size != display_size.size_rate)
            {
                init_display_size = display_size.size_rate;
                ChangeWindowSize(init_display_size);
            }
            display_size.Dispose();
        }

        //CheckBoxGrip
        private void CheckBoxGrip_Checked(object sender, RoutedEventArgs e)
        {
            ResizeMode = ResizeMode.CanResizeWithGrip;
        }
        private void CheckBoxGrip_Unchecked(object sender, RoutedEventArgs e)
        {
            ResizeMode = ResizeMode.NoResize;
        }

        //Set Parameters
        private void Set_parameters()
        {

            Properties.Settings.Default.AllowsTransparency_FLG = AllowsTransparency_FLG;
            Properties.Settings.Default.strech_FLG = strech_FLG;
            Properties.Settings.Default.continue_FLG = continue_FLG;
            Properties.Settings.Default.init_visibility = init_visibility;
            Properties.Settings.Default.topmost_FLG = topmost_FLG;
            Properties.Settings.Default.loop_FLG = loop_FLG;
            Init_display_size_calc();
            Properties.Settings.Default.init_display_size = init_display_size;

            Form_settings form_Settings = new Form_settings();
            form_Settings.ShowDialog();
            form_Settings.Dispose();

            libVlcDllsPath = Properties.Settings.Default.libVlcDllsPath;
            libVlcPluginsPath = Properties.Settings.Default.libVlcPluginsPath;

            if (AllowsTransparency_FLG != Properties.Settings.Default.AllowsTransparency_FLG)
            {
                MessageBox.Show
                    (
                        "Window Style is Changed. Apply next time.",
                        "CuteMP4 Information",
                        MessageBoxButton.OK,
                        MessageBoxImage.Information
                    );
                AllowsTransparency_FLG = Properties.Settings.Default.AllowsTransparency_FLG;
            }

            if (topmost_FLG != Properties.Settings.Default.topmost_FLG)
            {
                Topmost = !Topmost;
                topmost_FLG = Properties.Settings.Default.topmost_FLG;
            }
            if (strech_FLG != Properties.Settings.Default.strech_FLG)
            {
                strech_FLG = Properties.Settings.Default.strech_FLG;
                Stretch_Mode_Change(Properties.Settings.Default.strech_FLG);
            }

            continue_FLG = Properties.Settings.Default.continue_FLG;
            init_visibility = Properties.Settings.Default.init_visibility;
            loop_FLG = Properties.Settings.Default.loop_FLG;

            if (init_display_size != Properties.Settings.Default.init_display_size)
            {
                init_display_size = Properties.Settings.Default.init_display_size;
                ChangeWindowSize(Properties.Settings.Default.init_display_size);
            }
        }
 
        //**********************************************************************
        //  画面スリープ抑止
        //**********************************************************************
        /* http://d.hatena.ne.jp/aont/20100121/1264089608
       * アプリケーションによっては動作中、
       * スリープや画面停止をさせたくない物もあるでしょう、
       * それを実現する実装を備忘録として記載します
       * C#といってもC#のみでは実現不可なのでWinAPIを利用します、
       * なのでC言語ならもっとスマートに実装出来ます
       * Vista以前は【SetThreadExecutionState】を呼び出す事で阻止出来たのですが、
       * Vista以降は不可能になっています
       * なので定期的にマウス操作を加えることで阻止を実現します
       * マウス操作はある程度低レベルで実行しなければいけません、
       * なのでこれもWinAPIを使います
            //タイマー処理(画面スリープ抑止)
            //タイマー生成
            System.Windows.Forms.Timer timer = new System.Windows.Forms.Timer();
            //画面スリープ抑止
            timer.Tick += new EventHandler(OnTick_Timer);
            //30秒ごと処理
            timer.Interval = 30000;
            //タイマー開始
            timer.Start();
       */

        [FlagsAttribute]
        public enum ExecutionState : uint
        {
            // 関数が失敗した時の戻り値
            Null = 0,
            // スタンバイを抑止(Vista以降は効かない？)
            SystemRequired = 1,
            // 画面OFFを抑止
            DisplayRequired = 2,
            // 効果を永続させる。ほかオプションと併用する。
            Continuous = 0x80000000,
        }
        [DllImport("user32.dll")]
        extern static uint SendInput
          (
            // INPUT 構造体の数(イベント数)
            uint nInputs,
            // INPUT 構造体
            INPUT[] pInputs,
            // INPUT 構造体のサイズ
            int cbSize
           );
        [DllImport("kernel32.dll")]
        extern static ExecutionState SetThreadExecutionState(ExecutionState esFlags);
        // アンマネージ DLL 対応用 struct 記述宣言
        [StructLayout(LayoutKind.Sequential)]
        struct INPUT
        {
            // 0 = INPUT_MOUSE(デフォルト), 1 = INPUT_KEYBOARD
            public int type;
            // Note: struct の場合、デフォルト(パラメータなしの)コンストラクタは、
            //言語側で定義済みで、フィールドを 0 に初期化する
            public MOUSEINPUT mi;

        }

        // アンマネージ DLL 対応用 struct 記述宣言
        [StructLayout(LayoutKind.Sequential)]
        struct MOUSEINPUT
        {
            public int dx;
            public int dy;
            public int mouseData;  // amount of wheel movement
            public int dwFlags;
            public int time;  // time stamp for the event
            public IntPtr dwExtraInfo;
            // Note: struct の場合、デフォルト(パラメータなしの)コンストラクタは、
            //言語側で定義済みで、フィールドを 0 に初期化する
        }
        // dwFlags
        const int MOUSEEVENTF_MOVED = 0x0001;
        const int MOUSEEVENTF_LEFTDOWN = 0x0002;  // 左ボタン Down
        const int MOUSEEVENTF_LEFTUP = 0x0004;  // 左ボタン Up
        const int MOUSEEVENTF_RIGHTDOWN = 0x0008;  // 右ボタン Down
        const int MOUSEEVENTF_RIGHTUP = 0x0010;  // 右ボタン Up
        const int MOUSEEVENTF_MIDDLEDOWN = 0x0020;  // 中ボタン Down
        const int MOUSEEVENTF_MIDDLEUP = 0x0040;  // 中ボタン Up
        const int MOUSEEVENTF_WHEEL = 0x0080;
        const int MOUSEEVENTF_XDOWN = 0x0100;
        const int MOUSEEVENTF_XUP = 0x0200;
        const int MOUSEEVENTF_ABSOLUTE = 0x8000;
        const int screen_length = 0x10000;  // for MOUSEEVENTF_ABSOLUTE (この値は固定)
        //画面スリープ抑止
        public void OnTick_Timer(object sender, EventArgs e)
        {
            //画面暗転阻止 ※再生状況書き込みを追加 2017.09.10
            SetThreadExecutionState(ExecutionState.DisplayRequired);
            // ドラッグ操作の準備 (struct 配列の宣言)
            INPUT[] input = new INPUT[1];  // イベントを格納
            // ドラッグ操作の準備 (イベントの定義 = 相対座標へ移動)
            input[0].mi.dx = 0;  // 相対座標で0 つまり動かさない
            input[0].mi.dy = 0;  // 相対座標で0 つまり動かさない
            input[0].mi.dwFlags = MOUSEEVENTF_MOVED;
            // ドラッグ操作の実行 (イベントの生成)
            SendInput(1, input, Marshal.SizeOf(input[0]));
            //再生状況書き込み 追加 2017.09.10 ADD
            //Playlog(); 2017.10.01 DEL
            Task.Run(() => Playlog()); //2017.10.01 ADD
        }
    }
}
